﻿package {
	import flash.display.MovieClip;
	import flash.display.Graphics;
	import flash.events.Event;
	import flash.events.MouseEvent;
	import flash.geom.Point;
	import flash.geom.Rectangle;
	
	public class Pong extends MovieClip {
		const paddleW:Number = 20;
		const paddleL:Number = 100;
		const edgeOffset = 20;
		const p2Offset = stage.stageWidth - edgeOffset - paddleW;
		const gravity = .01;
		var active:Boolean;
		var p1:Paddle;
		var p2:Paddle;
		var p3:Paddle;
		var ground:Paddle;
		var bounds:Rectangle;
		var trail:Vector.<Point>;
		var trailSize:int = 20;
		public static var ball:Point;
		public static var ballVel:Point;
		public function Pong():void {
			trail = new Vector.<Point>(trailSize, true);
			stopGame();
			p1 = new Paddle(edgeOffset, mouseY, paddleW, paddleL, true);
			p2 = new Paddle(p2Offset, mouseY, paddleW, paddleL, true);
			p3 = new Paddle(mouseX, edgeOffset, paddleL, paddleW, true);
			ground = new Paddle(edgeOffset + paddleW, stage.stageHeight - edgeOffset,
								   p2Offset - edgeOffset - paddleW, edgeOffset, false);
			bounds = new Rectangle(0, 0, stage.stageWidth, stage.stageHeight);
			stage.addEventListener(Event.ENTER_FRAME, update);
		}
		public function update(evt:Event):void {
			p1.y = p2.y = mouseY - .5 * paddleL;
			p3.x = mouseX - .5 * paddleL;
			if (active) {
				updateBall();
			}
			graphics.clear();
			p1.drawTo(this);
			p2.drawTo(this);
			p3.drawTo(this);
			ground.drawTo(this);
			drawBall();
		}
		public function drawBall():void {
			graphics.beginFill(0x000000);
			graphics.drawCircle(ball.x, ball.y, 5);
			graphics.endFill();
			var lineAlpha:Number = .75;
			graphics.moveTo(ball.x, ball.y);
			for (var i:int = 1; i < trailSize; i++) {
				graphics.lineStyle(10, 0x000000, lineAlpha);
				graphics.lineTo(trail[i].x, trail[i].y);
				lineAlpha *= .875;
			}
		}
		public function updateBall():void {
			if (pointInsideRect(ball, bounds)) {
				ball = ball.add(ballVel);
				ballVel.y += gravity;
				p1.updateCollide();
				p2.updateCollide();
				p3.updateCollide();
				ground.updateCollide();
			} else {
				stopGame();
			}
			for (var i:int = trailSize - 1; i != 0; i--) {
				trail[i] = trail[i-1];
			}
			trail[0] = ball.clone();
		}
		public function startGame(evt:MouseEvent):void {
			active = true;
			stage.removeEventListener(MouseEvent.CLICK, startGame);
		}
		public function stopGame():void {
			active = false;
			stage.addEventListener(MouseEvent.CLICK, startGame);
			ball = new Point(.5 * stage.stageWidth, .5 * stage.stageHeight);
			ballVel = new Point(Math.random() + .5, Math.random() + .5);
			for(var i:int = 0; i < trailSize; i++) {
				trail[i] = ball.clone();
			}
		}
		public function pointInsideRect(p:Point, r:Rectangle):Boolean {
			return !((p.x < r.left) || (p.x > r.right) || (p.y < r.top) || (p.y > r.bottom));
		}
	}
}